# This file was generated automatically by Qleany's generator, edit at your own risk! 
# If you do, be careful to not overwrite it when you run the generator again.
Module Singles
{% for single_declaration in singles_to_declare_list -%}
{{ single_declaration }}
{% endfor %}
