// This file was generated automatically by Qleany's generator, edit at your own risk! 
// If you do, be careful to not overwrite it when you run the generator again.
#pragma once

#include "QtCore/qobjectdefs.h"
#include "QtCore/qtmetamacros.h"
#include "QtCore/QLatin1StringView"

using namespace Qt::Literals::StringLiterals;

namespace {{ application_cpp_domain_name }}::Entities::Entities
{
Q_NAMESPACE

enum EntityEnum
{
{%- for entity in entities_list %}
    {{ entity.name }}{% if not loop.last %},{% endif %}
{%- endfor %}
};
Q_ENUM_NS(EntityEnum);

} // namespace {{ application_cpp_domain_name }}::Entities::Entities