// This file was generated automatically by Qleany's generator, edit at your own risk! 
// If you do, be careful to not overwrite it when you run the generator again.
#include "entities_registration.h"
#include "controller_registration.h"
#include "mainwindow.h"
#include "persistence_registration.h"
#include <QApplication>

int main(int argc, char *argv[])
{

    QApplication a(argc, argv);
    a.setApplicationName("{{ application_name }}"_L1);
    a.setOrganizationName("{{ organisation_name }}"_L1);
    a.setOrganizationDomain("{{ organisation_domain }}"_L1);

    new {{ application_cpp_domain_name }}::Entities::EntitiesRegistration(&a);
    auto *persistenceRegistration = new {{ application_cpp_domain_name }}::Persistence::PersistenceRegistration(&a);
    new {{ application_cpp_domain_name }}::Controller::ControllerRegistration(&a, persistenceRegistration->repositoryProvider());

    MainWindow window;
    window.show();

    return a.exec();
}
