// This file was generated automatically by Qleany's generator, edit at your own risk! 
// If you do, be careful to not overwrite it when you run the generator again.

#pragma once

#include "{{ export_header_file }}"
#include <qleany/contracts/repository/interface_repository_provider.h>
#include <QObject>

namespace {{ application_cpp_domain_name }}::Controller
{

class {{ export }} ControllerRegistration : public QObject
{
    Q_OBJECT
  public:
    explicit ControllerRegistration(QObject *parent,
                                    Qleany::Contracts::Repository::InterfaceRepositoryProvider *repositoryProvider);
    ~ControllerRegistration();

  private:
};

} // namespace {{ application_cpp_domain_name }}::Controller
