# This file was generated automatically by Qleany's generator, edit at your own risk! 
# If you do, be careful to not overwrite it when you run the generator again.

add_subdirectory(real_imports)

# For integration in QT Design Studio project, you may have to replace
# ${APP_NAME} by ${CMAKE_PROJECT_NAME} or by the name of your project
target_link_libraries(${APP_NAME} PRIVATE
{{ application_name }}-qml-controllersplugin
{{ application_name }}-qml-modelsplugin
{{ application_name }}-qml-singlesplugin
)
