#pragma once
#include "{{ single.single_snake_name}}.h"
#include <QQmlEngine>

struct Foreign{{ single.single_pascal_name}}
{
    Q_GADGET
    QML_FOREIGN({{ application_cpp_domain_name }}::Presenter::{{ single.single_pascal_name}})
    QML_NAMED_ELEMENT({{ single.single_pascal_name}})
};
