# This file was generated automatically by Qleany's generator, edit at your own risk! 
# If you do, be careful to not overwrite it when you run the generator again.
Module Controllers
{% for singleton in singleton_list -%}
{{ singleton }}
{% endfor %}
