# SPDX-License-Identifier: CC0-1.0
# SPDX-FileCopyrightText: %{CURRENT_YEAR} %{AUTHOR} <%{EMAIL}>
[Desktop Entry]
Name={{ application_name }}
Comment={{ application_name }} Kirigami Application
Version=1.0
Exec={{ application_name }}
Icon=applications-development
Type=Application
Terminal=false
# Add an actual main category here (and possibly applicable additional ones)
#   https://specifications.freedesktop.org/menu-spec/latest/apa.html#main-category-registry
Categories=Qt;KDE;
