# This file was generated automatically by Qleany's generator, edit at your own risk! 
# If you do, be careful to not overwrite it when you run the generator again.

add_subdirectory(models)
add_subdirectory(controllers)
add_subdirectory(singles)
