{
    "id": "org.kde.{{ application_name_lower }}",
    "runtime": "org.kde.Platform",
    "runtime-version": "5.15",
    "sdk": "org.kde.Sdk",
    "command": "{{ application_name_lower }}",
    "tags": ["nightly"],
    "desktop-file-name-suffix": " (Nightly)",
    "finish-args": [
        "--share=ipc",
        "--share=network",
        "--socket=x11",
        "--socket=wayland",
        "--device=dri",
        "--filesystem=home"
    ],
    "separate-locales": false,

    "modules": [
        {
            "name": "{{ application_name_lower }}",
            "buildsystem": "cmake-ninja",
            "builddir": true,
            "sources": [ { "type": "dir", "path": ".", "skip": [".git"] } ]
        }
    ]
}

