<?xml version="1.0" encoding="utf-8"?>
<!--
  SPDX-License-Identifier: FSFAP
  SPDX-FileCopyrightText: %{CURRENT_YEAR} %{AUTHOR} <%{EMAIL}>
 -->
<component type="desktop-application">
  <id>org.kde.{{ application_name_lower }}</id>
  <name>{{ application_name }} Kirigami Application</name>
  <summary>A short summary describing what this software is about</summary>
  <metadata_license>A permissive license for this metadata, e.g. "FSFAP" + Update the SPDX tags above!</metadata_license>
  <project_license>The license of this software as SPDX string, e.g. "GPL-2.0-or-later"</project_license>
  <developer_name>The software vendor name, e.g. "ACME Corporation"</developer_name>
  <description>
    <p>Multiple paragraphs of long description, describing this software component.</p>
    <p>You can also use ordered and unordered lists:</p>
    <ul>
      <li>Feature 1</li>
      <li>Feature 2</li>
    </ul>
    <p>Keep in mind to XML-escape characters, and that this is not HTML markup.</p>
  </description>
</component>
