// This file was generated automatically by Qleany's generator, edit at your own risk! 
// If you do, be careful to not overwrite it when you run the generator again.
#pragma once
#include "{{ model.list_model_snake_name}}.h"
#include <QQmlEngine>

struct Foreign{{ model.list_model_pascal_name}}
{
    Q_GADGET
    QML_FOREIGN({{ application_cpp_domain_name }}::Presenter::{{ model.list_model_pascal_name}})
    QML_NAMED_ELEMENT({{ model.list_model_pascal_name}})
};
